# Form implementation generated from reading ui file 'dialogs/warning.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Warning(object):
    def setupUi(self, Warning):
        Warning.setObjectName("Warning")
        Warning.setWindowModality(QtCore.Qt.WindowModality.WindowModality.ApplicationModal)
        Warning.resize(400, 300)
        self.buttonBox = QtWidgets.QDialogButtonBox(Warning)
        self.buttonBox.setGeometry(QtCore.QRect(30, 250, 341, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.title = QtWidgets.QLabel(Warning)
        self.title.setGeometry(QtCore.QRect(20, 30, 361, 31))
        font = QtGui.QFont()
        font.setPointSize(24)
        self.title.setFont(font)
        self.title.setObjectName("title")
        self.message = QtWidgets.QLabel(Warning)
        self.message.setGeometry(QtCore.QRect(20, 80, 361, 151))
        self.message.setWordWrap(True)
        self.message.setObjectName("message")

        self.retranslateUi(Warning)
        self.buttonBox.accepted.connect(Warning.accept)
        self.buttonBox.rejected.connect(Warning.reject)
        QtCore.QMetaObject.connectSlotsByName(Warning)

    def retranslateUi(self, Warning):
        _translate = QtCore.QCoreApplication.translate
        Warning.setWindowTitle(_translate("Warning", "Dialog"))
        self.title.setText(_translate("Warning", "Warning"))
        self.message.setText(_translate("Warning", "Text here."))
