# Form implementation generated from reading ui file 'dialogs/latticetype.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_LatticeType(object):
    def setupUi(self, LatticeType):
        LatticeType.setObjectName("LatticeType")
        LatticeType.resize(215, 80)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(LatticeType.sizePolicy().hasHeightForWidth())
        LatticeType.setSizePolicy(sizePolicy)
        LatticeType.setSizeGripEnabled(False)
        self.verticalLayout = QtWidgets.QVBoxLayout(LatticeType)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(LatticeType)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.buttonBox = QtWidgets.QDialogButtonBox(LatticeType)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.No|QtWidgets.QDialogButtonBox.StandardButton.Yes)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(LatticeType)
        self.buttonBox.accepted.connect(LatticeType.accept)
        self.buttonBox.rejected.connect(LatticeType.reject)
        QtCore.QMetaObject.connectSlotsByName(LatticeType)

    def retranslateUi(self, LatticeType):
        _translate = QtCore.QCoreApplication.translate
        LatticeType.setWindowTitle(_translate("LatticeType", "Import"))
        self.label.setText(_translate("LatticeType", "Is this a square lattice design?"))
