# Form implementation generated from reading ui file 'dialogs/preferences.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Preferences(object):
    def setupUi(self, Preferences):
        Preferences.setObjectName("Preferences")
        Preferences.resize(465, 374)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Preferences.sizePolicy().hasHeightForWidth())
        Preferences.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(Preferences)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(Preferences)
        self.tabWidget.setObjectName("tabWidget")
        self.settings = QtWidgets.QWidget()
        self.settings.setObjectName("settings")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.settings)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.tabWidget_2 = QtWidgets.QTabWidget(self.settings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget_2.sizePolicy().hasHeightForWidth())
        self.tabWidget_2.setSizePolicy(sizePolicy)
        self.tabWidget_2.setObjectName("tabWidget_2")
        self.tab_honeycomb = QtWidgets.QWidget()
        self.tab_honeycomb.setObjectName("tab_honeycomb")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_honeycomb)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.formLayout_3 = QtWidgets.QFormLayout()
        self.formLayout_3.setFieldGrowthPolicy(QtWidgets.QFormLayout.FieldGrowthPolicy.FieldsStayAtSizeHint)
        self.formLayout_3.setContentsMargins(-1, 0, -1, 0)
        self.formLayout_3.setObjectName("formLayout_3")
        self.dimensionsLabel1 = QtWidgets.QLabel(self.tab_honeycomb)
        self.dimensionsLabel1.setObjectName("dimensionsLabel1")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.dimensionsLabel1)
        self.honeycombGLayout = QtWidgets.QGridLayout()
        self.honeycombGLayout.setContentsMargins(-1, 3, -1, -1)
        self.honeycombGLayout.setObjectName("honeycombGLayout")
        self.honeycombRowsSpinBox = QtWidgets.QSpinBox(self.tab_honeycomb)
        self.honeycombRowsSpinBox.setMinimum(6)
        self.honeycombRowsSpinBox.setMaximum(200)
        self.honeycombRowsSpinBox.setProperty("value", 30)
        self.honeycombRowsSpinBox.setObjectName("honeycombRowsSpinBox")
        self.honeycombGLayout.addWidget(self.honeycombRowsSpinBox, 0, 0, 1, 1)
        self.honeycombColsSpinBox = QtWidgets.QSpinBox(self.tab_honeycomb)
        self.honeycombColsSpinBox.setMinimum(6)
        self.honeycombColsSpinBox.setMaximum(200)
        self.honeycombColsSpinBox.setProperty("value", 32)
        self.honeycombColsSpinBox.setObjectName("honeycombColsSpinBox")
        self.honeycombGLayout.addWidget(self.honeycombColsSpinBox, 0, 1, 1, 1)
        self.hRowsLabel = QtWidgets.QLabel(self.tab_honeycomb)
        self.hRowsLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.hRowsLabel.setObjectName("hRowsLabel")
        self.honeycombGLayout.addWidget(self.hRowsLabel, 1, 0, 1, 1)
        self.hColsLabel = QtWidgets.QLabel(self.tab_honeycomb)
        self.hColsLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.hColsLabel.setObjectName("hColsLabel")
        self.honeycombGLayout.addWidget(self.hColsLabel, 1, 1, 1, 1)
        self.honeycombStepsSpinBox = QtWidgets.QSpinBox(self.tab_honeycomb)
        self.honeycombStepsSpinBox.setMinimum(2)
        self.honeycombStepsSpinBox.setMaximum(2000)
        self.honeycombStepsSpinBox.setSingleStep(1)
        self.honeycombStepsSpinBox.setProperty("value", 2)
        self.honeycombStepsSpinBox.setObjectName("honeycombStepsSpinBox")
        self.honeycombGLayout.addWidget(self.honeycombStepsSpinBox, 0, 2, 1, 1)
        self.hBasesLabel = QtWidgets.QLabel(self.tab_honeycomb)
        self.hBasesLabel.setObjectName("hBasesLabel")
        self.honeycombGLayout.addWidget(self.hBasesLabel, 1, 2, 1, 1)
        self.formLayout_3.setLayout(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.honeycombGLayout)
        self.verticalLayout_3.addLayout(self.formLayout_3)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/part/dlg_honeycomb"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.tabWidget_2.addTab(self.tab_honeycomb, icon, "")
        self.tab_square = QtWidgets.QWidget()
        self.tab_square.setObjectName("tab_square")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab_square)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setContentsMargins(-1, 0, -1, 0)
        self.formLayout_2.setObjectName("formLayout_2")
        self.dimensionsLabel2 = QtWidgets.QLabel(self.tab_square)
        self.dimensionsLabel2.setObjectName("dimensionsLabel2")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.dimensionsLabel2)
        self.squareGLayout = QtWidgets.QGridLayout()
        self.squareGLayout.setContentsMargins(-1, 3, -1, -1)
        self.squareGLayout.setObjectName("squareGLayout")
        self.squareRowsSpinBox = QtWidgets.QSpinBox(self.tab_square)
        self.squareRowsSpinBox.setMinimum(6)
        self.squareRowsSpinBox.setMaximum(200)
        self.squareRowsSpinBox.setProperty("value", 30)
        self.squareRowsSpinBox.setObjectName("squareRowsSpinBox")
        self.squareGLayout.addWidget(self.squareRowsSpinBox, 0, 0, 1, 1)
        self.squareColsSpinBox = QtWidgets.QSpinBox(self.tab_square)
        self.squareColsSpinBox.setMinimum(6)
        self.squareColsSpinBox.setMaximum(200)
        self.squareColsSpinBox.setProperty("value", 30)
        self.squareColsSpinBox.setObjectName("squareColsSpinBox")
        self.squareGLayout.addWidget(self.squareColsSpinBox, 0, 1, 1, 1)
        self.sRowsLabel = QtWidgets.QLabel(self.tab_square)
        self.sRowsLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.sRowsLabel.setObjectName("sRowsLabel")
        self.squareGLayout.addWidget(self.sRowsLabel, 1, 0, 1, 1)
        self.sColsLabel = QtWidgets.QLabel(self.tab_square)
        self.sColsLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.sColsLabel.setObjectName("sColsLabel")
        self.squareGLayout.addWidget(self.sColsLabel, 1, 1, 1, 1)
        self.squareStepsSpinBox = QtWidgets.QSpinBox(self.tab_square)
        self.squareStepsSpinBox.setMinimum(2)
        self.squareStepsSpinBox.setMaximum(2000)
        self.squareStepsSpinBox.setObjectName("squareStepsSpinBox")
        self.squareGLayout.addWidget(self.squareStepsSpinBox, 0, 2, 1, 1)
        self.sBasesLabel = QtWidgets.QLabel(self.tab_square)
        self.sBasesLabel.setObjectName("sBasesLabel")
        self.squareGLayout.addWidget(self.sBasesLabel, 1, 2, 1, 1)
        self.formLayout_2.setLayout(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.squareGLayout)
        self.verticalLayout_2.addLayout(self.formLayout_2)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/part/dlg_square"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.tabWidget_2.addTab(self.tab_square, icon1, "")
        self.verticalLayout_4.addWidget(self.tabWidget_2)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.FieldGrowthPolicy.FieldsStayAtSizeHint)
        self.formLayout.setObjectName("formLayout")
        self.autoscafLabel = QtWidgets.QLabel(self.settings)
        self.autoscafLabel.setObjectName("autoscafLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.autoscafLabel)
        self.autoScafComboBox = QtWidgets.QComboBox(self.settings)
        self.autoScafComboBox.setObjectName("autoScafComboBox")
        self.autoScafComboBox.addItem("")
        self.autoScafComboBox.addItem("")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.autoScafComboBox)
        self.zoomSpeedLabel = QtWidgets.QLabel(self.settings)
        self.zoomSpeedLabel.setObjectName("zoomSpeedLabel")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.zoomSpeedLabel)
        self.zoomSpeedSlider = QtWidgets.QSlider(self.settings)
        self.zoomSpeedSlider.setMinimumSize(QtCore.QSize(140, 0))
        self.zoomSpeedSlider.setMinimum(1)
        self.zoomSpeedSlider.setMaximum(100)
        self.zoomSpeedSlider.setSingleStep(1)
        self.zoomSpeedSlider.setProperty("value", 50)
        self.zoomSpeedSlider.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.zoomSpeedSlider.setInvertedControls(False)
        self.zoomSpeedSlider.setTickPosition(QtWidgets.QSlider.TickPosition.TicksBelow)
        self.zoomSpeedSlider.setTickInterval(0)
        self.zoomSpeedSlider.setObjectName("zoomSpeedSlider")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.zoomSpeedSlider)
        self.defaultToolComboBox = QtWidgets.QComboBox(self.settings)
        self.defaultToolComboBox.setLayoutDirection(QtCore.Qt.LayoutDirection.LeftToRight)
        self.defaultToolComboBox.setObjectName("defaultToolComboBox")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/pathtools/dlg_select"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.defaultToolComboBox.addItem(icon2, "")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/pathtools/dlg_pencil"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.defaultToolComboBox.addItem(icon3, "")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/pathtools/dlg_paint"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.defaultToolComboBox.addItem(icon4, "")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/pathtools/dlg_addseq"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.defaultToolComboBox.addItem(icon5, "")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.defaultToolComboBox)
        self.defaultToolLabel = QtWidgets.QLabel(self.settings)
        self.defaultToolLabel.setObjectName("defaultToolLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.defaultToolLabel)
        self.verticalLayout_4.addLayout(self.formLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(self.settings)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.RestoreDefaults)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_4.addWidget(self.buttonBox)
        self.tabWidget.addTab(self.settings, "")
        self.plugins = QtWidgets.QWidget()
        self.plugins.setObjectName("plugins")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.plugins)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.pluginTableWidget = QtWidgets.QTableWidget(self.plugins)
        self.pluginTableWidget.setObjectName("pluginTableWidget")
        self.pluginTableWidget.setColumnCount(1)
        self.pluginTableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.pluginTableWidget.setHorizontalHeaderItem(0, item)
        self.pluginTableWidget.horizontalHeader().setStretchLastSection(True)
        self.verticalLayout_5.addWidget(self.pluginTableWidget)
        self.addPluginButton = QtWidgets.QPushButton(self.plugins)
        self.addPluginButton.setObjectName("addPluginButton")
        self.verticalLayout_5.addWidget(self.addPluginButton)
        self.tabWidget.addTab(self.plugins, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.actionClose = QtGui.QAction(Preferences)
        self.actionClose.setObjectName("actionClose")

        self.retranslateUi(Preferences)
        self.tabWidget.setCurrentIndex(0)
        self.tabWidget_2.setCurrentIndex(0)
        self.defaultToolComboBox.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Preferences)

    def retranslateUi(self, Preferences):
        _translate = QtCore.QCoreApplication.translate
        Preferences.setWindowTitle(_translate("Preferences", "Preferences"))
        self.dimensionsLabel1.setText(_translate("Preferences", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Lucida Grande\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Default</p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">dimensions</p></body></html>"))
        self.hRowsLabel.setText(_translate("Preferences", "rows"))
        self.hColsLabel.setText(_translate("Preferences", "columns"))
        self.hBasesLabel.setText(_translate("Preferences", "bases (x21)"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_honeycomb), _translate("Preferences", "Honeycomb"))
        self.dimensionsLabel2.setText(_translate("Preferences", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Lucida Grande\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Default</p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">dimensions</p></body></html>"))
        self.sRowsLabel.setText(_translate("Preferences", "rows"))
        self.sColsLabel.setText(_translate("Preferences", "columns"))
        self.sBasesLabel.setText(_translate("Preferences", "bases (x32)"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_square), _translate("Preferences", "Square"))
        self.autoscafLabel.setText(_translate("Preferences", "Auto-scaffold style:"))
        self.autoScafComboBox.setItemText(0, _translate("Preferences", "Mid-seam"))
        self.autoScafComboBox.setItemText(1, _translate("Preferences", "Raster"))
        self.zoomSpeedLabel.setText(_translate("Preferences", "Mousewheel zoom speed:"))
        self.defaultToolComboBox.setItemText(0, _translate("Preferences", "Select Tool"))
        self.defaultToolComboBox.setItemText(1, _translate("Preferences", "Pencil Tool"))
        self.defaultToolComboBox.setItemText(2, _translate("Preferences", "Paint"))
        self.defaultToolComboBox.setItemText(3, _translate("Preferences", "Add Seq"))
        self.defaultToolLabel.setText(_translate("Preferences", "Default tool at startup:"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.settings), _translate("Preferences", "Settings"))
        item = self.pluginTableWidget.horizontalHeaderItem(0)
        item.setText(_translate("Preferences", "Path"))
        self.addPluginButton.setText(_translate("Preferences", "Add Plugin"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.plugins), _translate("Preferences", "Plugins"))
        self.actionClose.setText(_translate("Preferences", "Close"))
        self.actionClose.setShortcut(_translate("Preferences", "Ctrl+W"))
